/** 
 *   FIXME ....
 *
 *   (c) Copyright 2003 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 **/

#include <stdio.h>

#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include "encoders.h"
#include "iar.h"

int gray2bin(int bits, int binary);

int fd_local = -1;

int encoder_num[] = {RA_CASE, DEC_CASE};

int encoders_open(int ra, int dec)
{
	encoder_num[RA_CASE] = ra;
	encoder_num[DEC_CASE] = dec;

	fd_local = open("/dev/e5850",O_RDONLY);
	return fd_local;
}

int encoders_close(void)
{
	int ret = close(fd_local);
	fd_local = -1;
	return ret;
}

int encoders_buff_len(void)
{
	int len = -1;
	if(fd_local){
		ioctl(fd_local, E5850_GETBUFFLEN, &len);
		len = len >> 4;
	}
	return len;
}

int encoders_clear_buff(void)
{
	if(fd_local)
		return ioctl(fd_local, E5850_CLEARBUFF);
	else
		return -1;
}

int encoders_is_running(void)
{
	if(fd_local)
		return ioctl(fd_local, E5850_ISRUNNING);
	else
		return -1;
}

int encoders_stop(void)
{
	if(fd_local)
		return ioctl(fd_local, E5850_STOP);
	else
		return -1;
}
	
int encoders_start(void)
{
	if(fd_local)
		return ioctl(fd_local, E5850_START);
	else
		return -1;
}

int encoders_get_raw_data(struct encoders_data *d)
{
	int i;
	char buff[E5850_DATA_LEN];
	char *np = buff;
	char *endp;

	/* FIXME: bits of resolutions (14) */
	if ( read(fd_local, buff, E5850_DATA_LEN) != E5850_DATA_LEN)
		return -1;
	for(i=0;i<NR_ENC;i++){
		d->encoder[i] = 
			gray2bin(14, (int) strtol(np, &endp, 16 ));
		np = endp + 1;
	}

	return 1;
}


int get_encoder(int num)
{
	int res = -1;

	if(fd_local){
		res = ioctl(fd_local, E5850_GETNVAL, &num);
		/* FIXME: bits of resolutions (14) */
		if( res == 0 )
			return gray2bin(14,num);
		
	}
	return res;
}


int encoder_set_cw(int num)
{
	if(fd_local)
		return ioctl(fd_local, E5850_SETCW, &num);
	return -1;
}

int encoder_set_ccw(int num)
{
	if(fd_local)
		return ioctl(fd_local, E5850_SETCCW, &num);
	return -1;
}


int encoder_set_zero(int num)
{
	if(fd_local)
		return ioctl(fd_local, E5850_SETZERO, &num);

	return -1;
}



int gray2bin(int bits, int binary)
{
	int gray_code = 0;
	int bit_number;
	int bit = 0;
	
	for (bit_number = bits-1; bit_number >= 0; bit_number--) {
		bit ^= binary >> bit_number & 1;
		if(bit)
			gray_code |= (1 << bit_number);
	}
	return gray_code;
}
		

int sidereal_clock_running(void)
{
	if(fd_local)
		return ioctl(fd_local, E5850_SCLKRUNNING);
	return -1;
}
