/** 
 *   FIXME ....
 *
 *   Readline implementation for antena position system...
 *
 *   (c) Copyright 2003 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 **/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <malloc.h>


#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/errno.h>


#include <readline/readline.h>
#include <readline/history.h>

/* The names of functions that actually do the manipulation. */
int com_help __P((char *));
int com_quit __P((char *));

/* A structure which contains information on the commands this program
   can understand. */

typedef struct {
  char *name;			/* User printable name of the function. */
  rl_icpfunc_t *func;		/* Function to call to do the job. */
  char *doc;			/* Documentation for this function.  */
} COMMAND;

COMMAND commands[] = {
  { "help", com_help, "Display this text" },
  { "?", com_help, "Synonym for `help'" },
  { "quit", com_quit, "Quit ..." },
  { (char *)NULL, (rl_icpfunc_t *)NULL, (char *)NULL }
};

/* When non-zero, this means the user is done using this program. */
int done = 0;


char *dupstr(char *s) /* FIXME: Way not strdup ?? */ 
{
	char *r;
	
	r = (char *) malloc ((strlen(s) + 1)*sizeof(char));
	strcpy (r, s);
	return (r);
}



/* Generator function for command completion.  STATE lets us
   know whether to start from scratch; without any state
   (i.e. STATE == 0), then we start at the top of the list. */
char *command_generator (char *text, int state)
{ 
#if 1 /* FIXME */
	static int list_index, len;
	char *name;

	/* If this is a new word to complete, initialize now.  This
	   includes saving the length of TEXT for efficiency, and
	   initializing the index variable to 0. */
	if (!state) {
		list_index = 0;
		len = strlen (text);
	}

	/* Return the next name which partially matches from the
	   command list. */
	while ((name = commands[list_index].name)) {
		list_index++;
		if (strncmp (name, text, len) == 0)
			return (dupstr(name));
	}
#endif
	/* If no names matched, then return NULL. */
	return ((char *)NULL);
}


char **fileman_completion (const char *text, int start, int end)
{
	char **matches;

	matches = (char **)NULL;

	/* If this word is at the start of the line, then it is a
	   command to complete.  Otherwise it is the name of a file in
	   the current directory.  */
	if (start == 0)
	  matches = rl_completion_matches (text, 
					   (rl_compentry_func_t *) command_generator);
	
	return (matches);
}


/* Tell the GNU Readline library how to complete.  We want to try to
   complete on command names if this is the first word in the line, or
   on filenames if not. */
void initialize_readline (void)
{
	/* Allow conditional parsing of the ~/.inputrc file. */
	rl_readline_name = "antenad";
	
	/* Tell the completer that we want a crack first. */
	rl_attempted_completion_function = fileman_completion;
	
/*
 * Variable: const char * rl_terminal_name 
 *
 * The terminal type, used for initialization. If not set by the
 * application, Readline sets this to the value of the TERM
 * environment variable the first time it is called.
 **/

	printf("rl_terminal_name: %s\n",rl_terminal_name);

/*
 * Variable: const char * rl_readline_name
 * 
 * This variable is set to a unique name by each application using
 * Readline. The value allows conditional parsing of the inputrc file
 * (see section 1.3.2 Conditional Init Constructs).
 **/

/*
 * Variable: FILE * rl_instream
 *
 * The stdio stream from which Readline reads input. If NULL, Readline
 * defaults to stdin.
 **/

/*
 * Variable: FILE * rl_outstream
 *
 *  The stdio stream to which Readline performs output. If NULL,
 *  Readline defaults to stdout.
 **/

/*
 * Variable: rl_getc_func_t * rl_getc_function
 *
 * If non-zero, Readline will call indirectly through this pointer to
 * get a character from the input stream. By default, it is set to
 * rl_getc, the default Readline character input function (see section
 * 2.4.8 Character Input).
 **/

/*
 * Variable: rl_voidfunc_t * rl_redisplay_function If non-zero,
 *
 * Readline will call indirectly through this pointer to update the
 * display with the current contents of the editing buffer. By
 * default, it is set to rl_redisplay, the default Readline redisplay
 * function (see section 2.4.6 Redisplay).
 **/


}


/* Look up NAME as the name of a command, and return a pointer to that
   command.  Return a NULL pointer if NAME isn't a command name. */
COMMAND *find_command (char *name)
{
	int i;

	for (i = 0; commands[i].name; i++)
		if (strcmp (name, commands[i].name) == 0)
			return (&commands[i]);
	
	return ((COMMAND *)NULL);
}


/* Execute a command line. */
int execute_line (char *line)
{
  int i;
	COMMAND *command;
	char *word;

	/* Isolate the command word. */
	i = 0;
	while (line[i] && whitespace (line[i]))	i++;
	word = line + i;

	while (line[i] && !whitespace (line[i])) i++;

	if (line[i])
		line[i++] = '\0';

	command = find_command (word);

	if (!command) {
		fprintf (stderr, "%s: No such command for "".\n", word);
		return (-1);
	}

	/* Get argument to command, if any. */
	while (whitespace (line[i])) i++;

	word = line + i;

	/* Call the function. */
	return ((*(command->func)) (word));
}


/* Strip whitespace from the start and end of STRING.  Return a pointer
   into STRING. */
char * stripwhite (char *string)
{
	char *s, *t;

	for (s = string; whitespace (*s); s++);
    
	if (*s == 0)
		return (s);

	t = s + strlen (s) - 1;
	while (t > s && whitespace (*t))
		t--;
	*++t = '\0';
	
	return s;
}


/* The user wishes to quit using this program.  Just set DONE
   non-zero. */
int com_quit (char *arg)
{
	done = 1;
	return (0);
}



/* Print out help for ARG, or for all of the commands if ARG is not
   present. */
int com_help (char *arg)
{
	int i;
	int printed = 0;

	for (i = 0; commands[i].name; i++) {
		if (!*arg || (strcmp (arg, commands[i].name) == 0)) {
			printf ("%s\t\t%s.\n", commands[i].name, 
				commands[i].doc);
			printed++;
		}
	}

	if (!printed) {
		printf ("No commands match `%s'.  Possibilties are:\n", arg);

		for (i = 0; commands[i].name; i++) {
			/* Print in six columns. */
			if (printed == 6) {
				printed = 0;
				printf ("\n");
			}
			
			printf ("%s\t", commands[i].name);
			printed++;
		}

		if (printed)
			printf ("\n");
	}
	return (0);
}



int command_main(void)
{
	char *line, *s;

	initialize_readline ();	/* Bind our completer. */

	/* Loop reading and executing lines until the user quits. */
	for ( ; done == 0; ) {
		line = readline ("TEST: ");
		
		if (!line)
			break;
		
		/* Remove leading and trailing whitespace from the
		   line.  Then, if there is anything left, add it to
		   the history list and execute it. */
		s = stripwhite (line);

		if (*s)
		{
			add_history (s);
			execute_line (s);
		}
		
		free (line);
	}
	exit (0);
}

